************************************************************************
* Class wwBanner
******************
***    Author: Rick Strahl
***            (c) West Wind Technologies, 1996
***   Contact: (541) 386-2087  / rstrahl@west0wind.com
***  Modified: 07/25/96
***  Function: This module contains several classes that are
***            supplemental to Web Connection.
***            
***            wwBanner     -   A rotating banner class that
***                             supports different banners to be
***                             displayed on each page hit.
***
***           wwCounter     -   not implemented yet
************************************************************************
#INCLUDE WCONNECT.H


*** Sample Code 
#IF .T.
CLEAR 
o=CREATE("wwBanner",".\")

o.AddBanner("sinxc0003.gif","fsinxc0003.htm",1)
o.AddBanner("Image2.gif","Page2.htm",1)
o.AddBanner("Image3.gif","Page3.htm",1)
*o.DeleteBanner("sinxc0003.htm")

? "Sequential..."
For x=1 to 6
  ? o.GetBanner()
ENDFOR
?
? "Random..." 
For x=1 to 6
  ? o.GetBanner(-1)
Endfor


o.DeleteBanner("sinxc0003.htm")
o.DeleteBanner("Page2.htm")
o.DeleteBanner("Page3.htm")


#ENDIF


*************************************************************
DEFINE CLASS wwBanner AS Custom
*************************************************************
***    Author: Rick Strahl
***            (c) West Wind Technologies, 1996
***   Contact: (541) 386-2087  / rstrahl@west-wind.com
***  Modified: 07/22/96
***
***  Function: Banner class that allows displaying 
***            Image/HREF combos on a hit rotation
*************************************************************

*** Custom Properties
cBannerFile="wwBanners"

nCurrentBanner=1          
lAutoclose=.T.            && Close table in Destroy
cDataPath=""              && wwBanners.dbf dir (DOS)

*-*cImage=""                 
*-*cLink=""
*-*nOrder=0

*** This determines the script link stem of the HREF
*** that is placed into the HTML display table
*** wwBanners adds AddBanner and DeleteBanner to this
*** stem when creating the HREF links
cScriptRef="wwcgi.dll?maintain~"

*** Stock Properties

************************************************************************
* wwBanner :: Init
*********************************
***  Function: Initializes the datapath of where to find the
***            wwBanners table that contains the link/gif refs
***      Pass: lcDataPath  -   Path where the Banners table sits
************************************************************************
FUNCTION Init
LPARAMETER lcDataPath

THIS.cDataPath=IIF(type("lcDataPath")="C",lcDataPath,"")

ENDFUNC
* Init

************************************************************************
* wwBanner :: Destroy
*********************************
***  Function: Destructor - Close wwBanners file.
************************************************************************
FUNCTION Destroy

IF THIS.lAutoClose AND ;
   USED("wwBanners")
   USE IN wwBanners
ENDIF

ENDFUNC
* Destroy

************************************************************************
* wwBanner :: GetBanner
*********************************
***  Function: The actual Work Routine that retrieves a banner
***            HTML string.
***    Assume:
***      Pass: lnBanner  -  Banner Number to display
***                         Pass by reference to obtain next banner
***                         number that will be displayed.
***                         You can also Specify -1 to generate a
***                         random banner.
***            lcType     - Type of Banner to display. Acts as filter
***                         to allow multiple banners to be controlled
***                         as a individual or groups.
***    Return: Banner String
************************************************************************
FUNCTION GetBanner
LPARAMETERS lnBanner, lcType
LOCAL lnCount, lcOldAlias

lcOldAlias=ALIAS()

lnBanner=IIF(type("lnBanner")="N",lnBanner,THIS.nCurrentBanner)
lcType=IIF(type("lcType")="C",lcType,"  ")

=THIS.OpenTable()

SELECT *, Recno() AS RecNumber ;
   FROM (THIS.cDataPath+THIS.cBannerFile) ;
   WHERE type = lcType ;
   INTO CURSOR TBanners ;
   ORDER BY Order

*** Random Banner
IF lnBanner=-1
  =RAND(-1)
  lnBanner = INT( rand() * _Tally ) + 1
ENDIF

IF lnBanner=0
   lnBanner=1
ENDIF   
IF _TALLY<lnBanner or _TALLY<1
   *** Error Condition
   USE IN TBanners
   IF !EMPTY(lcOldAlias)
     SELE (lcOldAlias)
   ENDIF  
   RETURN ""
ENDIF

GOTO lnBanner

IF TBanners.MaxHits # 0 AND ;
   TBanners.Hits >= TBanners.MaxHits 
   lnBanner=THIS.GetNextBannerNumber(lnBanner)  
ENDIF     

lcOut=THIS.ImageHREF(TBanners.image,TBanners.link,TBanners.ImgTags)
IF !EMPTY(lcOut)
   REPLACE wwBanners.hits with wwBanners.hits+1 ;
           IN wwBanners ;
           FOR  recno()=Tbanners.recnumber
ENDIF   

lnBanner=THIS.GetNextBannerNumber(lnBanner)

USE IN TBanners

IF !EMPTY(lcOldAlias)
  SELE (lcOldAlias)
ENDIF  

RETURN lcOut
* GetBanner

************************************************************************
* wwBanner :: GetNextBannerNumber
*********************************
***  Function: Returns the next banner number. Checks for hit count
***            and wraps to front of list
***    Assume:
***    Return:
************************************************************************
PROTECTED FUNCTION GetNextBannerNumber
LPARAMETER lnBanner
LOCAL llExit, lnOldBanner

lnBanner=IIF(type("lnBanner")="N",lnBanner,1)
IF lnBanner=-1
   RETURN -1
ENDIF   

lnOldBanner=lnBanner
llExit=.F.

*** Loop until we find a good banner
*** or until we come back to the same one we started from
DO WHILE !llExit 
  SKIP 

  IF EOF()
    lnBanner=1
    GO TOP
  ELSE
    lnBanner=lnBanner+1
  ENDIF   
  
  IF MaxHits = 0 OR ;
     Hits <= MaxHits 
     llExit=.t.
  ENDIF
  
  *** No valid banners - use random instead
  IF lnBanner = lnOldBanner
     lnBanner=-1
     llExit=.T.
  ENDIF     
ENDDO  

THIS.nCurrentBanner=lnBanner

RETURN lnBanner

************************************************************************
* wwBanner :: ImageHREF
*********************************
***  Function: The actual Display Function. Returns an HREF string.
***    Assume: Separate to allow customization via Subclass
***      Pass: lcImage  -  Image to display - specify full ref'd path
***            lcLink   -  Link to go to
***            lcExtraImageTags -  Image Options like SIZE=xxx etc.
***    Return:
************************************************************************
FUNCTION ImageHREF
LPARAMETERS lcImage, lcLink, lcExtraTags

lcLink=IIF(type("lcLink")="C",lcLink,"")
lcExtraTags=IIF(type("lcExtraTags")="C",lcExtraTags,"")

lcOutput=[<IMG SRC="]+RTRIM(lcImage)+[" BORDER=0 ]+RTRIM(lcExtraTags)+[>]

IF !EMPTY(lcLink)
   RETURN [<A HREF="]+RTRIM(lcLink)+[">]+lcOutput+[</a>]
ENDIF

RETURN lcOutput   
* ImageHREF

************************************************************************
* wwBanner :: OpenTable
*********************************
***  Function: Opens and selects the wwBanner File
***            Creates the banner table if it doesn't exist
************************************************************************
FUNCTION OpenTable

IF !FILE(THIS.cDataPath+THIS.cBannerFile+".dbf")
  CREATE TABLE ( THIS.cDataPath+THIS.cBannerFile+".dbf" );
     (IMAGE     C (50),;
      LINK    C (128),;
      IMGTAGS C (40),;
      Order I ,;
      Type  C (15),;
      Hits  N (8,0) ,;
      MaxHits N (8,0) )
ELSE
   IF !USED("wwBanners")			
      USE (THIS.cDATAPATH+THIS.cBannerFile) IN 0 ALIAS WWBanners
   ENDIF

ENDIF

SELE wwBanners

ENDFUNC
* Open

************************************************************************
* wwBanner :: AddBanner
*********************************
***  Function: Adds an entry to the wwBanners table
***      Pass: lcImage     -  Image to display
***            lcLink      -  Link to run on click
***            lcImageTags -  Extra Image Tags
***            lnOrder     -  Display Order (ASC)
***            lcType      -  Group Id so you can have multiple
***                           separately controlled banners
***            lnHits      -  Counter that shows how often
***                           banner was hit
***            lnMaxHits   -  Won't show banner if hits reaches
***                           this value. 0 to disable MaxHits
***                           
***    Return: nothing
************************************************************************
FUNCTION AddBanner
LPARAMETERS lcImage, lcLink, lcImageTags, lnOrder, lcType, ;
            lnMaxHits

lcImage=IIF(type("lcImage")="C",lcImage,"")
lcLink=IIF(type("lcLink")="C",lcLink,"")
lcImageTags=IIF(type("lcImageTags")="C",lcImageTags,"")
lnOrder=IIF(type("lnOrder")="N",lnOrder,0)
lcType=IIF(type("lcType")="C",lcType,"")
lnMaxHits=IIF(type("lnMaxHits")="N",lnMaxHits,0)

THIS.OpenTable()

IF !EMPTY(lcImage) AND !EMPTY(lcLink)
   INSERT INTO (THIS.cBannerFile) ;
                (Image, Link, IMGTAGS, ORDER, Type, MaxHits) VALUES ;
                (lcImage,lcLink, lcImageTags, lnOrder, lcType,;
                 lnMaxHits)
ENDIF


ENDFUNC
* AddBanner

************************************************************************
* wwBanner :: DeleteBanner
*********************************
***  Function: Deletes an image entry based on an Image reference
***      Pass: lcImage   -  Banner to delete by image
***    Return: .t. or .f.
************************************************************************
FUNCTION DeleteBanner
LPARAMETERS lcImage

THIS.OpenTable()

LOCATE FOR UPPER(wwBanners.Image) = UPPER(lcImage)+" "
IF FOUND()
   DELETE
   IF OPENEXCLUSIVE(DBF())
      PACK
   ENDIF
   RETURN .T.
ENDIF

RETURN .F.
* DeleteBanner


ENDDEFINE



************************************************************************
FUNCTION OpenExclusive
**********************
***  Modified: 01/27/96
***  Function: Tries to open a table exclusively
***    Assume: Table name can't contain a file name.
***            Returns .F. for other reasons like file !found etc.
***      Pass: lcTable   -  Name of table to open exclusively
***    Return: .T. or .F.
************************************************************************
LPARAMETERS lcTable, lcAlias
LOCAL lcOldError, llError, llRetVal

lcTable=IIF(type("lcTable")="C",lcTable,"")
lcAlias=IIF(type("lcAlias")="C",lcAlias,JustStem(lcTable))

IF EMPTY(lcTable)
   RETURN .F.
ENDIF

lcOldError=ON("ERROR")
ON ERROR llError=.T.
llError=.F.

*** Use Exclusively to reindex and pack
IF !USED(lcAlias)
   USE (lcTable) EXCLUSIVE IN 0 ALIAS (lcAlias) 
ELSE
   SELE (lcAlias)
   USE (lcTable) EXCLUSIVE  ALIAS (lcAlias)
ENDIF

llRetVal=!llError
llError=.f.

*** Now try to re-open table as shared
IF !llRetVal
   USE (lcTable) IN 0
ENDIF   

*** Reset Error Handler
ON ERROR &lcOldError

RETURN llRetVal
*EOP OpenExclusive


************************************************************************
FUNCTION JustStem
*****************
***  Function: Returns just the Filename without extension.
***      Pass: lcFile  -  File name or filename and path
***    Return: file stem
*************************************************************************
LPARAMETER lcFile
lcFile=JustFname(lcFile)
lnDot=AT(".",lcFile)
IF lnDot < 2
   RETURN lcFile
ENDIF   
RETURN LEFT(lcFile,lnDot-1)

************************************************************************
FUNCTION JustFname
******************
***  Function: Returns just the filename portion of a path spec
***      Pass: lcFullPath
***    Return: Filename
*************************************************************************
LPARAMETER lcFilePath

IF type("lcFilePath")#"C"
   RETURN ""
ENDIF

lcFilePath=STRTRAN(TRIM(lcFilePath),"/","\")
lcFilePath=STRTRAN(lcFilePath,"\\","\")

lnLastSlash=RAT("\",lcFilePath)
If lnLastSlash=0
  *** Now check for Drive only
  lnLastSlash=RAT(":",lcFilePath)
ENDIF

*** No slashes - return full file name
IF lnLastSlash=0
   RETURN lcFilePath
ENDIF

*** No Filename in path if spec ends in slash
IF RIGHT(lcFilePath,1)="\"
   RETURN ""
ENDIF
   
RETURN SUBSTR(lcFilePath,lnLastSlash+1)
